#include<cstdio>
#include<algorithm>
#include<vector>
#include<queue>
#include<stack>
#include<set>
#include<map>
#include<cmath>
#include<cstdlib>
#include<string>
#include<cstring>
using namespace std;
const long double EPS = 1e-13;
const long double INF = 1e+10;
const long double PI = acos (-1);
int
sig (long double r)
{
  return (r < -EPS) ? -1 : (r > EPS) ? +1 : 0;
}

long double
ABS (long double a)
{
  return max (a, -a);
}

struct Pt
{
  long double x, y;
    Pt ()
  {
  }
  Pt (long double x, long double y):x (x), y (y)
  {
  }
  Pt operator+ (const Pt & a) const
  {
    return Pt (x + a.x, y + a.y);
  }
  Pt operator- (const Pt & a) const
  {
    return Pt (x - a.x, y - a.y);
  }
  Pt operator* (const Pt & a) const
  {
    return Pt (x * a.x - y * a.y, x * a.y + y * a.x);
  }
  Pt operator- () const
  {
    return Pt (-x, -y);
  }
  Pt operator* (const long double &k) const
  {
    return Pt (x * k, y * k);
  }
  Pt operator/ (const long double &k) const
  {
    return Pt (x / k, y / k);
  }
  long double ABS () const
  {
    return sqrt (x * x + y * y);
  }
  long double arg () const
  {
    return atan2 (y, x);
  }
  long double dot (const Pt & a) const
  {
    return x * a.x + y * a.y;
  }
  long double det (const Pt & a) const
  {
    return x * a.y - y * a.x;
  }
  bool operator== (const Pt & a) const
  {
    return (a - *this).ABS () < EPS;
  }
};
long double
tri (const Pt & a, const Pt & b, const Pt & c)
{
  return (b - a).det (c - a);
}

int
iSP (Pt a, Pt b, Pt c)
{
  int s = sig ((b - a).det (c - a));
  if (s)
    return s;
  if (sig ((b - a).dot (c - a)) < 0)
    return -2;
  if (sig ((a - b).dot (c - b)) < 0)
    return 2;
  return 0;
}

int
iLL (Pt a, Pt b, Pt c, Pt d)
{
  if (sig ((b - a).det (d - c)))
    return 1;
  if (sig ((b - a).det (c - a)))
    return 0;
  return -1;
}

Pt
pLL (Pt a, Pt b, Pt c, Pt d)
{
  b = b - a;
  d = d - c;
  return a + b * (c - a).det (d) / b.det (d);
}

Pt
hLP (Pt a, Pt b, Pt c)
{
  return pLL (a, b, c, c + (b - a) * Pt (0, 1));
}

bool
iSS (Pt a, Pt b, Pt c, Pt d)
{
  return (iSP (a, b, c) * iSP (a, b, d) <= 0
	  && iSP (c, d, a) * iSP (c, d, b) <= 0);
}

long double
dLP (Pt a, Pt b, Pt c)
{
  return ABS (tri (a, b, c)) / (b - a).ABS ();
}

long double
dLL (Pt a, Pt b, Pt c, Pt d)
{
  return iLL (a, b, c, d) ? 0 : dLP (a, b, c);
}

long double
dSP (Pt a, Pt b, Pt c)
{
  if (sig ((b - a).dot (c - a)) <= 0)
    return (c - a).ABS ();
  if (sig ((a - b).dot (c - b)) <= 0)
    return (c - b).ABS ();
  return ABS (tri (a, b, c)) / (b - a).ABS ();
}

long double
dSS (Pt a, Pt b, Pt c, Pt d)
{
  return iSS (a, b, c, d) ? 0 : min (min (dSP (a, b, c), dSP (a, b, d)),
				     min (dSP (c, d, a), dSP (c, d, b)));
}

int s_a[50], s_b[50], s_ab[50];
int
sAP (Pt a, Pt b, Pt c)
{
  return sig (a.det (c)) - sig (b.det (c)) - sig (a.det (b));
}

bool
iGSstrict (int n, Pt p[], Pt a, Pt b)
{
  int i;
  p[n] = p[0];
  p[n + 1] = p[1];
  for (i = 0; i <= n; i++)
    {
      s_a[i] = sig (tri (p[i], p[i + 1], a));
      s_b[i] = sig (tri (p[i], p[i + 1], b));
      s_ab[i] = sig (tri (a, b, p[i]));
    }
  for (i = 0; i < n; i++)
    {
      if (s_a[i] * s_b[i] < 0 && s_ab[i] * s_ab[i + 1] < 0)
	return 1;
    }
  for (i = 0; i < n; i++)
    {
      if (s_a[i] == 0 && s_b[i] > 0
	  && sig ((a - p[i]).dot (a - p[i + 1])) < 0)
	return 1;
      if (s_b[i] == 0 && s_a[i] > 0
	  && sig ((b - p[i]).dot (b - p[i + 1])) < 0)
	return 1;
    }
  for (i = 0; i < n; i++)
    if (s_ab[i + 1] == 0 && sig ((p[i + 1] - a).dot (p[i + 1] - b)) <= 0)
      {
	if (!(p[i + 1] == a)
	    && sAP (p[i + 2] - p[i + 1], p[i] - p[i + 1], a - p[i + 1]) > 0)
	  return 1;
	if (!(p[i + 1] == b)
	    && sAP (p[i + 2] - p[i + 1], p[i] - p[i + 1], b - p[i + 1]) > 0)
	  return 1;
      }
  return 0;
}

int wn;
Pt wa[50];
Pt W[50];
Pt E[50];
int ws, es;
long double ijk[1100];
int v[1100];
Pt var[1100];
long double nowbest = 999999999.9;
long double
calc (Pt s, Pt t, int side)
{
  for (int i = 0; i < 100; i++)
    {
      v[i] = 0;
      ijk[i] = 999999999.9;
    }
  int n;
  if (side == 0)
    n = ws + 2;
  else
    n = es + 2;
  if (side == 0)
    {
      for (int i = 0; i < ws; i++)
	var[i] = W[i];
      var[ws] = s;
      var[ws + 1] = t;
    }
  else
    {
      for (int i = 0; i < es; i++)
	var[i] = E[i];
      var[es] = s;
      var[es + 1] = t;
    }
  ijk[n - 2] = 0;
  for (int i = 0; i < n; i++)
    {
      long double tmp = 9999999999.9;
      int at = 0;
      for (int j = 0; j < n; j++)
	{
	  if (v[j])
	    continue;
	  if (tmp > ijk[j])
	    {
	      tmp = ijk[j];
	      at = j;
	    }
	}
      v[at] = 1;
      //printf("%d: %f\n",at,ijk[at]);
      for (int j = 0; j < n; j++)
	{
	  if (v[j])
	    continue;
	  if (ijk[j] < ijk[at] + (var[j] - var[at]).ABS ())
	    continue;
	  if ((var[j] - var[at]).ABS () < EPS)
	    {
	      ijk[j] = ijk[at];
	      continue;
	    }
	  if (iGSstrict (wn, wa, var[j], var[at]))
	    continue;
	  //printf("ok\n");
	  ijk[j] = ijk[at] + (var[j] - var[at]).ABS ();
	}
    }
  //printf("(%f, %f) -> (%f, %f): %f\n",s.x,s.y,t.x,t.y,ijk[n-1]);
  return ijk[n - 1];
}

int
main ()
{
  long double sx, sy, tx, ty;
  scanf ("%Lf%Lf%Lf%Lf", &sx, &sy, &tx, &ty);
  Pt s = Pt (sx, sy);
  Pt t = Pt (tx, ty);
  int a, b;
  scanf ("%d", &a);
  for (int i = 0; i < a; i++)
    {
      long double x, y;
      scanf ("%Lf%Lf", &x, &y);
      W[i] = Pt (x, y);
    }
  scanf ("%d", &b);
  for (int i = 0; i < b; i++)
    {
      long double x, y;
      scanf ("%Lf%Lf", &x, &y);
      E[i] = Pt (x, y);
    }
  ws = a;
  es = b;
  long double r1 = 999999999;
  vector < pair < Pt, Pt > >best;
  vector < pair < pair < Pt, Pt >, pair < Pt, Pt > > >bli;
  for (int i = 0; i < a - 1; i++)
    for (int j = 0; j < b - 1; j++)
      {
	long double t = dSS (W[i], W[i + 1], E[j], E[j + 1]);
	if (t < r1 - EPS)
	  {
	    r1 = t;
	  }
      }
  for (int i = 0; i < a - 1; i++)
    for (int j = 0; j < b - 1; j++)
      {
	long double t = dSS (W[i], W[i + 1], E[j], E[j + 1]);
	if (t > r1 + EPS)
	  continue;
	if (iLL (W[i], W[i + 1], E[j], E[j + 1]) == 1
	    || dLL (W[i], W[i + 1], E[j], E[j + 1]) < r1 - EPS)
	  {
	    if ((W[i] - E[j]).ABS () < r1 + EPS)
	      best.push_back (make_pair (W[i], E[j]));
	    if ((W[i] - E[j + 1]).ABS () < r1 + EPS)
	      best.push_back (make_pair (W[i], E[j + 1]));
	    if ((W[i + 1] - E[j]).ABS () < r1 + EPS)
	      best.push_back (make_pair (W[i + 1], E[j]));
	    if ((W[i + 1] - E[j + 1]).ABS () < r1 + EPS)
	      best.push_back (make_pair (W[i + 1], E[j + 1]));
	    if ((W[i] - hLP (E[j], E[j + 1], W[i])).ABS () < r1 + EPS
		&& iSP (E[j], hLP (E[j], E[j + 1], W[i]), E[j + 1]) == 2)
	      best.push_back (make_pair (W[i], hLP (E[j], E[j + 1], W[i])));
	    if ((W[i + 1] - hLP (E[j], E[j + 1], W[i + 1])).ABS () < r1 + EPS
		&& iSP (E[j], hLP (E[j], E[j + 1], W[i + 1]), E[j + 1]) == 2)
	      best.
		push_back (make_pair
			   (W[i + 1], hLP (E[j], E[j + 1], W[i + 1])));
	    if ((E[j] - hLP (W[i], W[i + 1], E[j])).ABS () < r1 + EPS
		&& iSP (W[i], hLP (W[i], W[i + 1], E[j]), W[i + 1]) == 2)
	      best.push_back (make_pair (hLP (W[i], W[i + 1], E[j]), E[j]));
	    if ((E[j + 1] - hLP (W[i], W[i + 1], E[j + 1])).ABS () < r1 + EPS
		&& iSP (W[i], hLP (W[i], W[i + 1], E[j + 1]), W[i + 1]) == 2)
	      best.
		push_back (make_pair
			   (hLP (W[i], W[i + 1], E[j + 1]), E[j + 1]));
	  }
	else if (iLL (W[i], W[i + 1], E[j], E[j + 1]) == 0)
	  {
	    vector < Pt > tmp;
	    tmp.push_back (hLP (W[i], W[i + 1], E[j]));
	    tmp.push_back (hLP (W[i], W[i + 1], E[j + 1]));
	    tmp.push_back (W[i]);
	    tmp.push_back (W[i + 1]);
	    for (int k = 0; k < 4; k++)
	      {
		for (int l = 0; l < 3; l++)
		  {
		    if (tmp[l].x * 11451 + tmp[l].y * 810 >
			tmp[l + 1].x * 11451 + tmp[l + 1].y * 810)
		      swap (tmp[l], tmp[l + 1]);
		  }
	      }
	    bli.
	      push_back (make_pair
			 (make_pair (tmp[1], tmp[2]),
			  make_pair (hLP (E[j], E[j + 1], tmp[1]),
				     hLP (E[j], E[j + 1], tmp[2]))));
	  }
      }
  for (int i = 0; i < b; i++)
    wa[wn++] = E[i];
  wa[wn++] = Pt (500, 100000000);
  for (int i = a - 1; i >= 0; i--)
    wa[wn++] = W[i];
  wa[wn++] = Pt (500, -100000000);
  long double r2 = 999999999;
  for (int i = 0; i < best.size (); i++)
    {
      //if((best[i].first-best[i].second).ABS()>r1+EPS)while(1);
      long double tmp =
	calc (s, best[i].first, 0) + calc (t, best[i].second, 1);
      r2 = min (r2, tmp);
    }
  for (int i = 0; i < bli.size (); i++)
    {
      long double left = 0;
      long double right = 1;
      // if((bli[i].first.first*m1+bli[i].first.second*(1.0-left)-bli[i].second.first*m1+bli[i].second.second*(1.0-left)).ABS()>r1+EPS)continue;
      // if((bli[i].first.first*m1+bli[i].first.second*(1.0-left)-bli[i].second.first*m1+bli[i].second.second*(1.0-left)).ABS()>r1+EPS)continue;
      for (int j = 0; j < 100; j++)
	{
	  long double m1 = (left * 2 + right) / 3;
	  long double m2 = (left + right * 2) / 3;
	  long double t1, t2;
	  // if((bli[i].first.first*m1+bli[i].first.second*(1.0-m1)-bli[i].second.first*m1+bli[i].second.second*(1.0-m1)).ABS()>r1+EPS)t1=999999999;
	  // if((bli[i].first.first*m2+bli[i].first.second*(1.0-m2)-bli[i].second.first*m2+bli[i].second.second*(1.0-m2)).ABS()>r1+EPS)t2=999999999;
	  t1 =
	    calc (s,
		  bli[i].first.first * m1 + bli[i].first.second * (1.0 - m1),
		  0) + calc (t,
			     bli[i].second.first * m1 +
			     bli[i].second.second * (1.0 - m1), 1);
	  t2 =
	    calc (s,
		  bli[i].first.first * m2 + bli[i].first.second * (1.0 - m2),
		  0) + calc (t,
			     bli[i].second.first * m2 +
			     bli[i].second.second * (1.0 - m2), 1);
	  if (t1 > t2)
	    left = m1;
	  else
	    right = m2;
	  r2 = min (r2, min (t1, t2));
	}
    }
  printf ("%.12Lf %.12Lf\n", r1, r1 + r2);
}
